@@ -29,7 +29,6 @@ import urlparse
 import hmac
 import binascii
 import httplib2
-from types import ListType
 
 try:
     from urlparse import parse_qs, parse_qsl
@@ -328,9 +327,17 @@ class Request(dict):
  
     def get_normalized_parameters(self):
         """Return a string that contains the parameters that must be signed."""
-        # 1.0a/9.1.1 states that kvp must be sorted by key, then by value
-        items = [(k, v if type(v) != ListType else sorted(v)) for k,v in sorted(self.items()) if k != 'oauth_signature']
-        encoded_str = urllib.urlencode(items, True)
+        items = []
+        for key, value in self.iteritems():
+            if key == 'oauth_signature':
+                continue
+            # 1.0a/9.1.1 states that kvp must be sorted by key, then by value,
+            # so we unpack sequence values into multiple items for sorting.
+            if hasattr(value, '__iter__'):
+                items.extend((key, item) for item in value)
+            else:
+                items.append((key, value))
+        encoded_str = urllib.urlencode(sorted(items))
         # Encode signature parameters per Oauth Core 1.0 protocol
         # spec draft 7, section 3.6
         # (http://tools.ietf.org/html/draft-hammer-oauth-07#section-3.6)
@@ -585,7 +592,13 @@ class Client(httplib2.Http):
             parameters = dict(parse_qsl(body))
         elif method == "GET":
             parsed = urlparse.urlparse(uri)
-            parameters = parse_qsl(parsed.query)
+
+            try:
+                query = parsed.query
+            except AttributeError:
+                query = parsed[4]
+
+            parameters = parse_qsl(query)     
         else:
             parameters = None
 
@@ -665,11 +678,11 @@ class SignatureMethod_HMAC_SHA1(SignatureMethod):
 
         # HMAC object.
         try:
-            import hashlib # 2.5
-            hashed = hmac.new(key, raw, hashlib.sha1)
+            import hashlib.sha1 as sha # 2.5
         except ImportError:
             import sha # Deprecated
-            hashed = hmac.new(key, raw, sha)
+
+        hashed = hmac.new(key, raw, sha)
 
         # Calculate the digest base 64.
         return binascii.b2a_base64(hashed.digest())[:-1]
